'***************************************  EtherCAT Setup Program  ***************************************
' File:			EC_SETUP.PRG
' Purpose:		EtherCAT setup example
' Version:		V1.0.0.0
' History:		11:57:54 27/09/2017 - V1.0.0.0 Created
' Author:		AUTOMATICALLY GENERATED by EC_SETUP_Gen.py
' Description:	This script demonstrates how to setup the EtherCAT module, which function/subroutines to invoke,
'				and in what order.
'
'********************************************************************************************************

' module global variables
common shared EC_SETUP_PRG_error_code as long = 0
common shared EC_SETUP_PRG_error_str as string = ""
common shared numOfSlaves as long = 0
common shared num_Of_Motion_Drives as long = 0

common shared ERR_EC_SETUP as error "ERROR SETTING UP ETHERCAT. CHECK EC_SETUP_PRG_error_code/EC_SETUP_PRG_error_STR" 20764

dim shared Note_PLL_Error_Retry as note "PLL Error in the drives. Restarting EC_SETUP.PRG" 20300

Program

	dim Counts_Per_Revolution as long = 10000  '  Set to 10000 by default for a generic drive
	dim ec_setup_iterations as long = 1
	dim retVal as long = 0
	dim retStr as string = ""
	dim drive_addr as long = 0
	dim PLL_Error as long = 0


	OnError

		Catch ERR_EC_SETUP.num
			' EC_SETUP_PRG_error_code/str are already occupied with correct data

		Catch Else

			EC_SETUP_PRG_error_code = EC_SETUP.PRG.ERRORNUMBER
			EC_SETUP_PRG_error_str = EC_SETUP.PRG.ERROR
			throw ERR_EC_SETUP

	End OnError

	retStr = VesExecute("CONFIG_PRG_SUCCESS = 0")

START_EC_SETUP_PRG:

	numOfSlaves = EC_ETHERCAT_INIT  '  Initialize EtherCAT according to a certain field bus, and type of slave addressing

	' Map PDO entries of the devices
	if PDO_Mapping_Device_1 = -1 then
		throw ERR_EC_SETUP
	end if

	if PDO_Mapping_Device_2 = -1 then
		throw ERR_EC_SETUP
	end if

	if PDO_Mapping_Device_3 = -1 then
		throw ERR_EC_SETUP
	end if

	if PDO_Mapping_Device_4 = -1 then
		throw ERR_EC_SETUP
	end if

	if PDO_Mapping_Device_5 = -1 then
		throw ERR_EC_SETUP
	end if

	if PDO_Mapping_Device_6 = -1 then
		throw ERR_EC_SETUP
	end if

	num_Of_Motion_Drives = 6

	' Validate that expected EtherCAT topology exists
	if EC_SLAVES_INFO[1]->vendor_id <> 0x2E1 or EC_SLAVES_INFO[1]->product_code <> 0x0 then
		EC_SETUP_PRG_error_code = 1001
		EC_SETUP_PRG_error_str = "MISMATCH IN VENDOR ID or PRODUCT CODE"
		throw ERR_EC_SETUP
	end if

	if EC_SLAVES_INFO[2]->vendor_id <> 0x2E1 or EC_SLAVES_INFO[2]->product_code <> 0x0 then
		EC_SETUP_PRG_error_code = 1037
		EC_SETUP_PRG_error_str = "MISMATCH IN VENDOR ID or PRODUCT CODE"
		throw ERR_EC_SETUP
	end if

	if EC_SLAVES_INFO[3]->vendor_id <> 0x2E1 or EC_SLAVES_INFO[3]->product_code <> 0x0 then
		EC_SETUP_PRG_error_code = 1069
		EC_SETUP_PRG_error_str = "MISMATCH IN VENDOR ID or PRODUCT CODE"
		throw ERR_EC_SETUP
	end if

	if EC_SLAVES_INFO[4]->vendor_id <> 0x2E1 or EC_SLAVES_INFO[4]->product_code <> 0x0 then
		EC_SETUP_PRG_error_code = 1105
		EC_SETUP_PRG_error_str = "MISMATCH IN VENDOR ID or PRODUCT CODE"
		throw ERR_EC_SETUP
	end if

	if EC_SLAVES_INFO[5]->vendor_id <> 0x2E1 or EC_SLAVES_INFO[5]->product_code <> 0x0 then
		EC_SETUP_PRG_error_code = 1141
		EC_SETUP_PRG_error_str = "MISMATCH IN VENDOR ID or PRODUCT CODE"
		throw ERR_EC_SETUP
	end if

	if EC_SLAVES_INFO[6]->vendor_id <> 0x2E1 or EC_SLAVES_INFO[6]->product_code <> 0x0 then
		EC_SETUP_PRG_error_code = 1173
		EC_SETUP_PRG_error_str = "MISMATCH IN VENDOR ID or PRODUCT CODE"
		throw ERR_EC_SETUP
	end if

	retVal = EC_RESCAN_SLAVES
	sleep 500

	retVal = CHECK_TOPOLOGY  '  Check the EtherCAT topology for miswiring - throws error

	EC_ADDRESS_BY_ALIAS = 0 ' Set to zero -> alias addressing is position based
	call EC_CREATE_MASTER

	print "Found ";numOfSlaves;" slaves"

	call EC_SET_BUS_CYCLETIME( 1000*4)  '  For a cycle time of 500 us, add to FWCONFIG: hpetfreq = 2000

	retVal = EC_SET_NEW_STATIC_DISTANCE(300) 
	'****************************************************************
	'																*
	'				PRE - OPERATIONAL CONFIGURATION					*
	' Configure each slave according to Vendor ID and Product Code  *
	'																*
	'****************************************************************


	' Invoke device 1 installation function 
	if INSTALL_DEVICE_1 = -1 then
		throw ERR_EC_SETUP
	end if

	' Invoke device 2 installation function 
	if INSTALL_DEVICE_2 = -1 then
		throw ERR_EC_SETUP
	end if

	' Invoke device 3 installation function 
	if INSTALL_DEVICE_3 = -1 then
		throw ERR_EC_SETUP
	end if

	' Invoke device 4 installation function 
	if INSTALL_DEVICE_4 = -1 then
		throw ERR_EC_SETUP
	end if

	' Invoke device 5 installation function 
	if INSTALL_DEVICE_5 = -1 then
		throw ERR_EC_SETUP
	end if

	' Invoke device 6 installation function 
	if INSTALL_DEVICE_6 = -1 then
		throw ERR_EC_SETUP
	end if



	'****************************************************************
	'																*
	'					USER PREOP CONFIGURATION					*
	' Add code to EC_USER_PREOP_CONFIG(byval drive_addr as long)	*
	'																*
	'****************************************************************
	'retVal = EC_USER_PREOP_CONFIG(drive_addr)

	' Create PDOs mapping, set bus cycle time, sync clocks between the master and the slaves,
	' raise slaves to OP-Mode and sets master's opmode to operational
	Try

		call EC_STARTMASTER

	Catch EC_MASTER_PLL_ERROR.num   '  EC_MASTER_PLL_ERROR

		PLL_Error = 1

	End Try

	' In case of PLL error in the drives we make up to 5 attempts to restart EC_SETUP
	if PLL_Error = 1 then

		if ec_setup_iterations < 6 then

			ec_setup_iterations = ec_setup_iterations + 1
			print "PLL Error in the drives. Restarting EC_SETUP.PRG attempt #: ";ec_setup_iterations
			Logger Note_PLL_Error_Retry
			call ec_stopmaster
			sleep 3000
			goto START_EC_SETUP_PRG

		else

			call ec_stopmaster
			throw EC_MASTER_PLL_ERROR

		end if

	end if


	'****************************************************************
	'																*
	'					OPERATIONAL CONFIGURATION					*
	' Configure each slave according to Vendor ID and Product Code	*
	'																*
	'****************************************************************
	for drive_addr = 1 to numOfSlaves

		' SAFEOP state configuration - Add code to EC_USER_SAFEOP_CONFIG(byval drive_addr as long)
		'retVal = EC_USER_SAFEOP_CONFIG(drive_addr)

		' OP state configuration - Add code to EC_USER_OP_CONFIG(byval drive_addr as long)
		'retVal = EC_USER_OP_CONFIG(drive_addr)

		if ETSLAVES[drive_addr]->et_slavetype = ECAT_MOTIONSLAVE then

			' Attempt to clear faults in the drives
			Try

				call EC_CLEAR_FAULTS(drive_addr)

			Catch Else

				' if clear faults failed due to PLL Sync Error, wait a little longer and retry
				if EC_SDO_READ(drive_addr, 0x603f, 0) = 0x7386 then

					printu "PLL Error in drive #. Sleeping 5 seconds and retrying to clear fault";drive_addr
					sleep 5000

					call EC_CLEAR_FAULTS(drive_addr)

				end if

			End Try

		end if

	next

	if OpSDO_Config_Device_1 = -1 then
		throw ERR_EC_SETUP
	end if

	if EC_IS_PDO(1 , 0x6060, 0) then
		call EC_PDO_WRITE(1, 0x6060, 0,8)
	else
		call EC_SDO_WRITE(1 , 0x6060, 0, 8,8)
	end if

	if OpSDO_Config_Device_2 = -1 then
		throw ERR_EC_SETUP
	end if

	if EC_IS_PDO(2 , 0x6060, 0) then
		call EC_PDO_WRITE(2, 0x6060, 0,8)
	else
		call EC_SDO_WRITE(2 , 0x6060, 0, 8,8)
	end if

	if OpSDO_Config_Device_3 = -1 then
		throw ERR_EC_SETUP
	end if

	if EC_IS_PDO(3 , 0x6060, 0) then
		call EC_PDO_WRITE(3, 0x6060, 0,8)
	else
		call EC_SDO_WRITE(3 , 0x6060, 0, 8,8)
	end if

	if OpSDO_Config_Device_4 = -1 then
		throw ERR_EC_SETUP
	end if

	if EC_IS_PDO(4 , 0x6060, 0) then
		call EC_PDO_WRITE(4, 0x6060, 0,8)
	else
		call EC_SDO_WRITE(4 , 0x6060, 0, 8,8)
	end if

	if OpSDO_Config_Device_5 = -1 then
		throw ERR_EC_SETUP
	end if

	if EC_IS_PDO(5 , 0x6060, 0) then
		call EC_PDO_WRITE(5, 0x6060, 0,8)
	else
		call EC_SDO_WRITE(5 , 0x6060, 0, 8,8)
	end if

	if OpSDO_Config_Device_6 = -1 then
		throw ERR_EC_SETUP
	end if

	if EC_IS_PDO(6 , 0x6060, 0) then
		call EC_PDO_WRITE(6, 0x6060, 0,8)
	else
		call EC_SDO_WRITE(6 , 0x6060, 0, 8,8)
	end if

	' Assign drive addresses to axes
	A1.busnumber = 0
	A1.dadd = 1
	A2.busnumber = 0
	A2.dadd = 2
	A3.busnumber = 0
	A3.dadd = 3
	A4.busnumber = 0
	A4.dadd = 4
	A5.busnumber = 0
	A5.dadd = 5
	A6.busnumber = 0
	A6.dadd = 6

	print "EtherCAT Setup Finished Successfully !"

EC_SETUP_PRG_DONE:

End Program


' Invoke configuration functions for device 1
Function INSTALL_DEVICE_1 as long

	dim PNUM as long

	INSTALL_DEVICE_1 = 0

	if Set_PDOe_To_AxProps_Device_1 = -1 then
		INSTALL_DEVICE_1 = -1
		goto INSTALL_DEVICE_1_DONE
	end if

	if SDO_Config_Device_1 = -1 then
		INSTALL_DEVICE_1 = -1
		goto INSTALL_DEVICE_1_DONE
	end if

	' Calculate Feedback Resolution
	PNUM = EC_SDO_READ(1, 0x6092, 1)
	if PNUM < 10000 then
		print "WARNING: PNUM is smaller than 10000. Low resolution might cause noises"
	end if
	call EC_USE_SLAVE(1, 0x2E1, 0x0, "ECAT_AXIS_SLAVE", TRUE)

	call EC_ASSOCIATE_PDO_TO_SYS_DIO(1, 0x60FD, 0, 1000, 11, 16)
	call EC_ASSOCIATE_PDO_TO_SYS_DIO(1, 0x60FE, 1, 1000, 6, 16)

INSTALL_DEVICE_1_DONE:

End Function


' Create PDO entries mapping of device 1
Function PDO_Mapping_Device_1 as long

	PDO_Mapping_Device_1 = 0

	' Zero number of entries in sync manager 2
	if Execute_VES("call EC_SDO_WRITE(1, EC_SYNC_MANAGER_1C12, 0, 8, 0)", 1003) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set PDOs in sync manager 2
	if Execute_VES("call EC_SDO_WRITE(1, EC_SYNC_MANAGER_1C12, 1, 16, 0x1600)", 1003) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	if Execute_VES("call EC_SDO_WRITE(1, EC_SYNC_MANAGER_1C12, 2, 16, 0x1601)", 1003) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	if Execute_VES("call EC_SDO_WRITE(1, EC_SYNC_MANAGER_1C12, 3, 16, 0x1603)", 1003) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set number of PDOs in the sync manager
	if Execute_VES("call EC_SDO_WRITE(1, EC_SYNC_MANAGER_1C12, 0, 8, 3)", 1003) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Zero number of PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(1, 0x1600, 0, 8, 0)", 1004) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(1, 0x1600, 1, 32, 0x60400010)", 1005) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	if Execute_VES("call EC_SDO_WRITE(1, 0x1600, 2, 32, 0x60600008)", 1006) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set number of PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(1, 0x1600, 0, 8, 2)", 1004) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Zero number of PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(1, 0x1601, 0, 8, 0)", 1007) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(1, 0x1601, 1, 32, 0x607A0020)", 1008) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	if Execute_VES("call EC_SDO_WRITE(1, 0x1601, 2, 32, 0x60810020)", 1009) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set number of PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(1, 0x1601, 0, 8, 2)", 1007) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Zero number of PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(1, 0x1603, 0, 8, 0)", 1010) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(1, 0x1603, 1, 32, 0x60FE0120)", 1011) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	if Execute_VES("call EC_SDO_WRITE(1, 0x1603, 2, 32, 0x60710010)", 1012) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	if Execute_VES("call EC_SDO_WRITE(1, 0x1603, 3, 32, 0x60B20010)", 1013) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set number of PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(1, 0x1603, 0, 8, 3)", 1010) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Zero number of entries in sync manager 3
	if Execute_VES("call EC_SDO_WRITE(1, EC_SYNC_MANAGER_1C13, 0, 8, 0)", 1014) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set PDOs in sync manager 3
	if Execute_VES("call EC_SDO_WRITE(1, EC_SYNC_MANAGER_1C13, 1, 16, 0x1A00)", 1014) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	if Execute_VES("call EC_SDO_WRITE(1, EC_SYNC_MANAGER_1C13, 2, 16, 0x1A01)", 1014) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	if Execute_VES("call EC_SDO_WRITE(1, EC_SYNC_MANAGER_1C13, 3, 16, 0x1A02)", 1014) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	if Execute_VES("call EC_SDO_WRITE(1, EC_SYNC_MANAGER_1C13, 4, 16, 0x1A03)", 1014) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set number of PDOs in the sync manager
	if Execute_VES("call EC_SDO_WRITE(1, EC_SYNC_MANAGER_1C13, 0, 8, 4)", 1014) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Zero number of PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(1, 0x1A00, 0, 8, 0)", 1015) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(1, 0x1A00, 1, 32, 0x60410010)", 1016) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	if Execute_VES("call EC_SDO_WRITE(1, 0x1A00, 2, 32, 0x60610008)", 1017) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	if Execute_VES("call EC_SDO_WRITE(1, 0x1A00, 3, 32, 0x60640020)", 1018) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set number of PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(1, 0x1A00, 0, 8, 3)", 1015) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Zero number of PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(1, 0x1A01, 0, 8, 0)", 1019) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(1, 0x1A01, 1, 32, 0x60640020)", 1020) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set number of PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(1, 0x1A01, 0, 8, 1)", 1019) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Zero number of PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(1, 0x1A02, 0, 8, 0)", 1021) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(1, 0x1A02, 1, 32, 0x60740010)", 1022) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	if Execute_VES("call EC_SDO_WRITE(1, 0x1A02, 2, 32, 0x20F20010)", 1023) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	if Execute_VES("call EC_SDO_WRITE(1, 0x1A02, 3, 32, 0x20F90010)", 1024) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	if Execute_VES("call EC_SDO_WRITE(1, 0x1A02, 4, 32, 0x60780010)", 1025) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set number of PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(1, 0x1A02, 0, 8, 4)", 1021) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Zero number of PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(1, 0x1A03, 0, 8, 0)", 1026) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(1, 0x1A03, 1, 32, 0x60FD0020)", 1027) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	if Execute_VES("call EC_SDO_WRITE(1, 0x1A03, 2, 32, 0x20B60020)", 1028) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	if Execute_VES("call EC_SDO_WRITE(1, 0x1A03, 3, 32, 0x60F40020)", 1029) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	' Set number of PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(1, 0x1A03, 0, 8, 3)", 1026) <> 0 then
		PDO_Mapping_Device_1 = -1
		goto ERR_PDO_DEVICE_1
	end if

	ETSLAVES[1]->et_opmode = 8

ERR_PDO_DEVICE_1:

End Function


' Configure device 1 via SDO writes
Function SDO_Config_Device_1 as long

	SDO_Config_Device_1 = 0

	print "PREOP configuration via SDO of device 1"
	print "Set MICRO_INTERPOLATION in device 1"
	if Execute_VES("call EC_SDO_WRITE(1, 0x60C0, 0, 16, 2)", 1030) <> 0 then
		SDO_Config_Device_1 = -1
		goto ERR_SDO_DEVICE_1
	end if

	print "Set PNUM in device 1"
	if Execute_VES("call EC_SDO_WRITE(1, 0x6092, 1, 32, 1000000)", 1031) <> 0 then
		SDO_Config_Device_1 = -1
		goto ERR_SDO_DEVICE_1
	end if

	print "Set PDEN in device 1"
	if Execute_VES("call EC_SDO_WRITE(1, 0x6092, 2, 32, 1)", 1032) <> 0 then
		SDO_Config_Device_1 = -1
		goto ERR_SDO_DEVICE_1
	end if

	print "Set GearRatio_MotorRevs in device 1"
	if Execute_VES("call EC_SDO_WRITE(1, 0x6091, 1, 32, 1)", 1033) <> 0 then
		SDO_Config_Device_1 = -1
		goto ERR_SDO_DEVICE_1
	end if

	print "Set GearRatio_ShaftRevs in device 1"
	if Execute_VES("call EC_SDO_WRITE(1, 0x6091, 2, 32, 1)", 1034) <> 0 then
		SDO_Config_Device_1 = -1
		goto ERR_SDO_DEVICE_1
	end if

	print "Set Smoothing in device 1"
	if Execute_VES("call EC_SDO_WRITE(1, 0x2109, 0, 16, 0)", 1035) <> 0 then
		SDO_Config_Device_1 = -1
		goto ERR_SDO_DEVICE_1
	end if

ERR_SDO_DEVICE_1:

End Function


' Configure device 1 via SDO writes
Function OpSDO_Config_Device_1 as long

	OpSDO_Config_Device_1 = 0

	print "OP configuration via SDO of device 1"
	print "Set DIGITAL_OUTPUTS_MASK in device 1"
	if Execute_VES("call EC_SDO_WRITE(1, 0x60FE, 2, 32, 0x3F0000)", 1036) <> 0 then
		OpSDO_Config_Device_1 = -1
		goto OpERR_SDO_DEVICE_1
	end if

OpERR_SDO_DEVICE_1:

End Function


' Associate PDO entries of device 1 with axis properties
Function Set_PDOe_To_AxProps_Device_1 as long

	Set_PDOe_To_AxProps_Device_1 = 0

	print "Associating between PDO entries of device 1 and axis properties"
	call EC_SET_CTRL_OBJECT(EC_BUSID, 1, EC_CTRL_INDEX_6040h, EC_CTRL_SUBINDEX_00h)

	call EC_SET_STATUS_OBJECT(EC_BUSID, 1, EC_STATUS_INDEX_6041h, EC_STATUS_SUBINDEX_00h)

	call EC_SET_PCMD_OBJECT(EC_BUSID, 1, EC_PCMD_INDEX_607Ah, EC_PCMD_SUBINDEX_00h)

	call EC_SET_PFB_OBJECT(EC_BUSID, 1, EC_PFB_INDEX_6064h, EC_PFB_SUBINDEX_00h)

	call EC_SET_VCMD_OBJECT(EC_BUSID, 1, EC_VCMD_INDEX_6081h, EC_VCMD_SUBINDEX_00h)

	call EC_SET_VADDCMD_OBJECT(EC_BUSID, 1, EC_VADDCMD_INDEX_60B1h, EC_VADDCMD_SUBINDEX_00h)

	call EC_SET_TRQCMD_OBJECT(EC_BUSID, 1, EC_TRQCMD_INDEX_6071h, EC_TRQCMD_SUBINDEX_00h)

	call EC_SET_TRQFBK_OBJECT(EC_BUSID, 1, EC_TRQFBK_INDEX_6078h, EC_TRQFBK_SUBINDEX_00h)

	call EC_SET_TRQDMND_OBJECT(EC_BUSID, 1, EC_TRQDMND_INDEX_6074h, EC_TRQDMND_SUBINDEX_00h)

	call EC_SET_TRQADDCMD_OBJECT(EC_BUSID, 1, EC_TRQADDCMD_INDEX_60B2h, EC_TRQADDCMD_SUBINDEX_00h)

	call EC_SET_PEXT_OBJECT(EC_BUSID, 1, EC_PEXT_INDEX_20B6h, EC_PEXT_SUBINDEX_00h)

ERR_PDOE_AXPROP_DEVICE_1:

End Function


' Invoke configuration functions for device 2
Function INSTALL_DEVICE_2 as long

	dim PNUM as long

	INSTALL_DEVICE_2 = 0

	if Set_PDOe_To_AxProps_Device_2 = -1 then
		INSTALL_DEVICE_2 = -1
		goto INSTALL_DEVICE_2_DONE
	end if

	if SDO_Config_Device_2 = -1 then
		INSTALL_DEVICE_2 = -1
		goto INSTALL_DEVICE_2_DONE
	end if

	' Calculate Feedback Resolution
	PNUM = EC_SDO_READ(2, 0x6092, 1)
	if PNUM < 10000 then
		print "WARNING: PNUM is smaller than 10000. Low resolution might cause noises"
	end if
	call EC_USE_SLAVE(2, 0x2E1, 0x0, "ECAT_AXIS_SLAVE", TRUE)

	call EC_ASSOCIATE_PDO_TO_SYS_DIO(2, 0x60FD, 0, 2000, 11, 16)
	call EC_ASSOCIATE_PDO_TO_SYS_DIO(2, 0x60FE, 1, 2000, 6, 16)

INSTALL_DEVICE_2_DONE:

End Function


' Create PDO entries mapping of device 2
Function PDO_Mapping_Device_2 as long

	PDO_Mapping_Device_2 = 0

	' Zero number of entries in sync manager 2
	if Execute_VES("call EC_SDO_WRITE(2, EC_SYNC_MANAGER_1C12, 0, 8, 0)", 1039) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set PDOs in sync manager 2
	if Execute_VES("call EC_SDO_WRITE(2, EC_SYNC_MANAGER_1C12, 1, 16, 0x1600)", 1039) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	if Execute_VES("call EC_SDO_WRITE(2, EC_SYNC_MANAGER_1C12, 2, 16, 0x1601)", 1039) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	if Execute_VES("call EC_SDO_WRITE(2, EC_SYNC_MANAGER_1C12, 3, 16, 0x1603)", 1039) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set number of PDOs in the sync manager
	if Execute_VES("call EC_SDO_WRITE(2, EC_SYNC_MANAGER_1C12, 0, 8, 3)", 1039) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Zero number of PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(2, 0x1600, 0, 8, 0)", 1040) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(2, 0x1600, 1, 32, 0x60400010)", 1041) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	if Execute_VES("call EC_SDO_WRITE(2, 0x1600, 2, 32, 0x60600008)", 1042) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set number of PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(2, 0x1600, 0, 8, 2)", 1040) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Zero number of PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(2, 0x1601, 0, 8, 0)", 1043) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(2, 0x1601, 1, 32, 0x607A0020)", 1044) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	if Execute_VES("call EC_SDO_WRITE(2, 0x1601, 2, 32, 0x60810020)", 1045) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set number of PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(2, 0x1601, 0, 8, 2)", 1043) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Zero number of PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(2, 0x1603, 0, 8, 0)", 1046) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(2, 0x1603, 1, 32, 0x60FE0120)", 1047) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set number of PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(2, 0x1603, 0, 8, 1)", 1046) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Zero number of entries in sync manager 3
	if Execute_VES("call EC_SDO_WRITE(2, EC_SYNC_MANAGER_1C13, 0, 8, 0)", 1048) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set PDOs in sync manager 3
	if Execute_VES("call EC_SDO_WRITE(2, EC_SYNC_MANAGER_1C13, 1, 16, 0x1A00)", 1048) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	if Execute_VES("call EC_SDO_WRITE(2, EC_SYNC_MANAGER_1C13, 2, 16, 0x1A01)", 1048) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	if Execute_VES("call EC_SDO_WRITE(2, EC_SYNC_MANAGER_1C13, 3, 16, 0x1A02)", 1048) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	if Execute_VES("call EC_SDO_WRITE(2, EC_SYNC_MANAGER_1C13, 4, 16, 0x1A03)", 1048) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set number of PDOs in the sync manager
	if Execute_VES("call EC_SDO_WRITE(2, EC_SYNC_MANAGER_1C13, 0, 8, 4)", 1048) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Zero number of PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(2, 0x1A00, 0, 8, 0)", 1049) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(2, 0x1A00, 1, 32, 0x60410010)", 1050) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	if Execute_VES("call EC_SDO_WRITE(2, 0x1A00, 2, 32, 0x60610008)", 1051) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set number of PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(2, 0x1A00, 0, 8, 2)", 1049) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Zero number of PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(2, 0x1A01, 0, 8, 0)", 1052) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(2, 0x1A01, 1, 32, 0x60640020)", 1053) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set number of PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(2, 0x1A01, 0, 8, 1)", 1052) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Zero number of PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(2, 0x1A02, 0, 8, 0)", 1054) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(2, 0x1A02, 1, 32, 0x60740010)", 1055) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	if Execute_VES("call EC_SDO_WRITE(2, 0x1A02, 2, 32, 0x20F20010)", 1056) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	if Execute_VES("call EC_SDO_WRITE(2, 0x1A02, 3, 32, 0x20F90010)", 1057) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set number of PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(2, 0x1A02, 0, 8, 3)", 1054) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Zero number of PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(2, 0x1A03, 0, 8, 0)", 1058) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(2, 0x1A03, 1, 32, 0x60FD0020)", 1059) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	if Execute_VES("call EC_SDO_WRITE(2, 0x1A03, 2, 32, 0x20B60020)", 1060) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	if Execute_VES("call EC_SDO_WRITE(2, 0x1A03, 3, 32, 0x60F40020)", 1061) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	' Set number of PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(2, 0x1A03, 0, 8, 3)", 1058) <> 0 then
		PDO_Mapping_Device_2 = -1
		goto ERR_PDO_DEVICE_2
	end if

	ETSLAVES[2]->et_opmode = 8

ERR_PDO_DEVICE_2:

End Function


' Configure device 2 via SDO writes
Function SDO_Config_Device_2 as long

	SDO_Config_Device_2 = 0

	print "PREOP configuration via SDO of device 2"
	print "Set MICRO_INTERPOLATION in device 2"
	if Execute_VES("call EC_SDO_WRITE(2, 0x60C0, 0, 16, 2)", 1062) <> 0 then
		SDO_Config_Device_2 = -1
		goto ERR_SDO_DEVICE_2
	end if

	print "Set PNUM in device 2"
	if Execute_VES("call EC_SDO_WRITE(2, 0x6092, 1, 32, 1000000)", 1063) <> 0 then
		SDO_Config_Device_2 = -1
		goto ERR_SDO_DEVICE_2
	end if

	print "Set PDEN in device 2"
	if Execute_VES("call EC_SDO_WRITE(2, 0x6092, 2, 32, 1)", 1064) <> 0 then
		SDO_Config_Device_2 = -1
		goto ERR_SDO_DEVICE_2
	end if

	print "Set GearRatio_MotorRevs in device 2"
	if Execute_VES("call EC_SDO_WRITE(2, 0x6091, 1, 32, 1)", 1065) <> 0 then
		SDO_Config_Device_2 = -1
		goto ERR_SDO_DEVICE_2
	end if

	print "Set GearRatio_ShaftRevs in device 2"
	if Execute_VES("call EC_SDO_WRITE(2, 0x6091, 2, 32, 1)", 1066) <> 0 then
		SDO_Config_Device_2 = -1
		goto ERR_SDO_DEVICE_2
	end if

	print "Set Smoothing in device 2"
	if Execute_VES("call EC_SDO_WRITE(2, 0x2109, 0, 16, 0)", 1067) <> 0 then
		SDO_Config_Device_2 = -1
		goto ERR_SDO_DEVICE_2
	end if

ERR_SDO_DEVICE_2:

End Function


' Configure device 2 via SDO writes
Function OpSDO_Config_Device_2 as long

	OpSDO_Config_Device_2 = 0

	print "OP configuration via SDO of device 2"
	print "Set DIGITAL_OUTPUTS_MASK in device 2"
	if Execute_VES("call EC_SDO_WRITE(2, 0x60FE, 2, 32, 0x3F0000)", 1068) <> 0 then
		OpSDO_Config_Device_2 = -1
		goto OpERR_SDO_DEVICE_2
	end if

OpERR_SDO_DEVICE_2:

End Function


' Associate PDO entries of device 2 with axis properties
Function Set_PDOe_To_AxProps_Device_2 as long

	Set_PDOe_To_AxProps_Device_2 = 0

	print "Associating between PDO entries of device 2 and axis properties"
	call EC_SET_CTRL_OBJECT(EC_BUSID, 2, EC_CTRL_INDEX_6040h, EC_CTRL_SUBINDEX_00h)

	call EC_SET_STATUS_OBJECT(EC_BUSID, 2, EC_STATUS_INDEX_6041h, EC_STATUS_SUBINDEX_00h)

	call EC_SET_PCMD_OBJECT(EC_BUSID, 2, EC_PCMD_INDEX_607Ah, EC_PCMD_SUBINDEX_00h)

	call EC_SET_PFB_OBJECT(EC_BUSID, 2, EC_PFB_INDEX_6064h, EC_PFB_SUBINDEX_00h)

	call EC_SET_VCMD_OBJECT(EC_BUSID, 2, EC_VCMD_INDEX_6081h, EC_VCMD_SUBINDEX_00h)

	call EC_SET_VADDCMD_OBJECT(EC_BUSID, 2, EC_VADDCMD_INDEX_60B1h, EC_VADDCMD_SUBINDEX_00h)

	call EC_SET_TRQCMD_OBJECT(EC_BUSID, 2, EC_TRQCMD_INDEX_6071h, EC_TRQCMD_SUBINDEX_00h)

	call EC_SET_TRQFBK_OBJECT(EC_BUSID, 2, EC_TRQFBK_INDEX_6078h, EC_TRQFBK_SUBINDEX_00h)

	call EC_SET_TRQDMND_OBJECT(EC_BUSID, 2, EC_TRQDMND_INDEX_6074h, EC_TRQDMND_SUBINDEX_00h)

	call EC_SET_TRQADDCMD_OBJECT(EC_BUSID, 2, EC_TRQADDCMD_INDEX_60B2h, EC_TRQADDCMD_SUBINDEX_00h)

	call EC_SET_PEXT_OBJECT(EC_BUSID, 2, EC_PEXT_INDEX_20B6h, EC_PEXT_SUBINDEX_00h)

ERR_PDOE_AXPROP_DEVICE_2:

End Function


' Invoke configuration functions for device 3
Function INSTALL_DEVICE_3 as long

	dim PNUM as long

	INSTALL_DEVICE_3 = 0

	if Set_PDOe_To_AxProps_Device_3 = -1 then
		INSTALL_DEVICE_3 = -1
		goto INSTALL_DEVICE_3_DONE
	end if

	if SDO_Config_Device_3 = -1 then
		INSTALL_DEVICE_3 = -1
		goto INSTALL_DEVICE_3_DONE
	end if

	' Calculate Feedback Resolution
	PNUM = EC_SDO_READ(3, 0x6092, 1)
	if PNUM < 10000 then
		print "WARNING: PNUM is smaller than 10000. Low resolution might cause noises"
	end if
	call EC_USE_SLAVE(3, 0x2E1, 0x0, "ECAT_AXIS_SLAVE", TRUE)

	call EC_ASSOCIATE_PDO_TO_SYS_DIO(3, 0x60FD, 0, 3000, 11, 16)
	call EC_ASSOCIATE_PDO_TO_SYS_DIO(3, 0x60FE, 1, 3000, 6, 16)

INSTALL_DEVICE_3_DONE:

End Function


' Create PDO entries mapping of device 3
Function PDO_Mapping_Device_3 as long

	PDO_Mapping_Device_3 = 0

	' Zero number of entries in sync manager 2
	if Execute_VES("call EC_SDO_WRITE(3, EC_SYNC_MANAGER_1C12, 0, 8, 0)", 1071) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set PDOs in sync manager 2
	if Execute_VES("call EC_SDO_WRITE(3, EC_SYNC_MANAGER_1C12, 1, 16, 0x1600)", 1071) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	if Execute_VES("call EC_SDO_WRITE(3, EC_SYNC_MANAGER_1C12, 2, 16, 0x1601)", 1071) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	if Execute_VES("call EC_SDO_WRITE(3, EC_SYNC_MANAGER_1C12, 3, 16, 0x1603)", 1071) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set number of PDOs in the sync manager
	if Execute_VES("call EC_SDO_WRITE(3, EC_SYNC_MANAGER_1C12, 0, 8, 3)", 1071) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Zero number of PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(3, 0x1600, 0, 8, 0)", 1072) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(3, 0x1600, 1, 32, 0x60400010)", 1073) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	if Execute_VES("call EC_SDO_WRITE(3, 0x1600, 2, 32, 0x60600008)", 1074) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set number of PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(3, 0x1600, 0, 8, 2)", 1072) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Zero number of PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(3, 0x1601, 0, 8, 0)", 1075) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(3, 0x1601, 1, 32, 0x607A0020)", 1076) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	if Execute_VES("call EC_SDO_WRITE(3, 0x1601, 2, 32, 0x60810020)", 1077) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set number of PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(3, 0x1601, 0, 8, 2)", 1075) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Zero number of PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(3, 0x1603, 0, 8, 0)", 1078) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(3, 0x1603, 1, 32, 0x60FE0120)", 1079) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	if Execute_VES("call EC_SDO_WRITE(3, 0x1603, 2, 32, 0x60710010)", 1080) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	if Execute_VES("call EC_SDO_WRITE(3, 0x1603, 3, 32, 0x60B20010)", 1081) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set number of PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(3, 0x1603, 0, 8, 3)", 1078) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Zero number of entries in sync manager 3
	if Execute_VES("call EC_SDO_WRITE(3, EC_SYNC_MANAGER_1C13, 0, 8, 0)", 1082) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set PDOs in sync manager 3
	if Execute_VES("call EC_SDO_WRITE(3, EC_SYNC_MANAGER_1C13, 1, 16, 0x1A00)", 1082) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	if Execute_VES("call EC_SDO_WRITE(3, EC_SYNC_MANAGER_1C13, 2, 16, 0x1A01)", 1082) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	if Execute_VES("call EC_SDO_WRITE(3, EC_SYNC_MANAGER_1C13, 3, 16, 0x1A02)", 1082) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	if Execute_VES("call EC_SDO_WRITE(3, EC_SYNC_MANAGER_1C13, 4, 16, 0x1A03)", 1082) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set number of PDOs in the sync manager
	if Execute_VES("call EC_SDO_WRITE(3, EC_SYNC_MANAGER_1C13, 0, 8, 4)", 1082) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Zero number of PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(3, 0x1A00, 0, 8, 0)", 1083) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(3, 0x1A00, 1, 32, 0x60410010)", 1084) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	if Execute_VES("call EC_SDO_WRITE(3, 0x1A00, 2, 32, 0x60610008)", 1085) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	if Execute_VES("call EC_SDO_WRITE(3, 0x1A00, 3, 32, 0x60640020)", 1086) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set number of PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(3, 0x1A00, 0, 8, 3)", 1083) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Zero number of PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(3, 0x1A01, 0, 8, 0)", 1087) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(3, 0x1A01, 1, 32, 0x60640020)", 1088) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set number of PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(3, 0x1A01, 0, 8, 1)", 1087) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Zero number of PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(3, 0x1A02, 0, 8, 0)", 1089) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(3, 0x1A02, 1, 32, 0x60740010)", 1090) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	if Execute_VES("call EC_SDO_WRITE(3, 0x1A02, 2, 32, 0x20F20010)", 1091) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	if Execute_VES("call EC_SDO_WRITE(3, 0x1A02, 3, 32, 0x20F90010)", 1092) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	if Execute_VES("call EC_SDO_WRITE(3, 0x1A02, 4, 32, 0x60780010)", 1093) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set number of PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(3, 0x1A02, 0, 8, 4)", 1089) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Zero number of PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(3, 0x1A03, 0, 8, 0)", 1094) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(3, 0x1A03, 1, 32, 0x60FD0020)", 1095) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	if Execute_VES("call EC_SDO_WRITE(3, 0x1A03, 2, 32, 0x20B60020)", 1096) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	if Execute_VES("call EC_SDO_WRITE(3, 0x1A03, 3, 32, 0x60F40020)", 1097) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	' Set number of PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(3, 0x1A03, 0, 8, 3)", 1094) <> 0 then
		PDO_Mapping_Device_3 = -1
		goto ERR_PDO_DEVICE_3
	end if

	ETSLAVES[3]->et_opmode = 8

ERR_PDO_DEVICE_3:

End Function


' Configure device 3 via SDO writes
Function SDO_Config_Device_3 as long

	SDO_Config_Device_3 = 0

	print "PREOP configuration via SDO of device 3"
	print "Set MICRO_INTERPOLATION in device 3"
	if Execute_VES("call EC_SDO_WRITE(3, 0x60C0, 0, 16, 2)", 1098) <> 0 then
		SDO_Config_Device_3 = -1
		goto ERR_SDO_DEVICE_3
	end if

	print "Set PNUM in device 3"
	if Execute_VES("call EC_SDO_WRITE(3, 0x6092, 1, 32, 1000000)", 1099) <> 0 then
		SDO_Config_Device_3 = -1
		goto ERR_SDO_DEVICE_3
	end if

	print "Set PDEN in device 3"
	if Execute_VES("call EC_SDO_WRITE(3, 0x6092, 2, 32, 1)", 1100) <> 0 then
		SDO_Config_Device_3 = -1
		goto ERR_SDO_DEVICE_3
	end if

	print "Set GearRatio_MotorRevs in device 3"
	if Execute_VES("call EC_SDO_WRITE(3, 0x6091, 1, 32, 1)", 1101) <> 0 then
		SDO_Config_Device_3 = -1
		goto ERR_SDO_DEVICE_3
	end if

	print "Set GearRatio_ShaftRevs in device 3"
	if Execute_VES("call EC_SDO_WRITE(3, 0x6091, 2, 32, 1)", 1102) <> 0 then
		SDO_Config_Device_3 = -1
		goto ERR_SDO_DEVICE_3
	end if

	print "Set Smoothing in device 3"
	if Execute_VES("call EC_SDO_WRITE(3, 0x2109, 0, 16, 0)", 1103) <> 0 then
		SDO_Config_Device_3 = -1
		goto ERR_SDO_DEVICE_3
	end if

ERR_SDO_DEVICE_3:

End Function


' Configure device 3 via SDO writes
Function OpSDO_Config_Device_3 as long

	OpSDO_Config_Device_3 = 0

	print "OP configuration via SDO of device 3"
	print "Set DIGITAL_OUTPUTS_MASK in device 3"
	if Execute_VES("call EC_SDO_WRITE(3, 0x60FE, 2, 32, 0x3F0000)", 1104) <> 0 then
		OpSDO_Config_Device_3 = -1
		goto OpERR_SDO_DEVICE_3
	end if

OpERR_SDO_DEVICE_3:

End Function


' Associate PDO entries of device 3 with axis properties
Function Set_PDOe_To_AxProps_Device_3 as long

	Set_PDOe_To_AxProps_Device_3 = 0

	print "Associating between PDO entries of device 3 and axis properties"
	call EC_SET_CTRL_OBJECT(EC_BUSID, 3, EC_CTRL_INDEX_6040h, EC_CTRL_SUBINDEX_00h)

	call EC_SET_STATUS_OBJECT(EC_BUSID, 3, EC_STATUS_INDEX_6041h, EC_STATUS_SUBINDEX_00h)

	call EC_SET_PCMD_OBJECT(EC_BUSID, 3, EC_PCMD_INDEX_607Ah, EC_PCMD_SUBINDEX_00h)

	call EC_SET_PFB_OBJECT(EC_BUSID, 3, EC_PFB_INDEX_6064h, EC_PFB_SUBINDEX_00h)

	call EC_SET_VCMD_OBJECT(EC_BUSID, 3, EC_VCMD_INDEX_6081h, EC_VCMD_SUBINDEX_00h)

	call EC_SET_VADDCMD_OBJECT(EC_BUSID, 3, EC_VADDCMD_INDEX_60B1h, EC_VADDCMD_SUBINDEX_00h)

	call EC_SET_TRQCMD_OBJECT(EC_BUSID, 3, EC_TRQCMD_INDEX_6071h, EC_TRQCMD_SUBINDEX_00h)

	call EC_SET_TRQFBK_OBJECT(EC_BUSID, 3, EC_TRQFBK_INDEX_6078h, EC_TRQFBK_SUBINDEX_00h)

	call EC_SET_TRQDMND_OBJECT(EC_BUSID, 3, EC_TRQDMND_INDEX_6074h, EC_TRQDMND_SUBINDEX_00h)

	call EC_SET_TRQADDCMD_OBJECT(EC_BUSID, 3, EC_TRQADDCMD_INDEX_60B2h, EC_TRQADDCMD_SUBINDEX_00h)

	call EC_SET_PEXT_OBJECT(EC_BUSID, 3, EC_PEXT_INDEX_20B6h, EC_PEXT_SUBINDEX_00h)

ERR_PDOE_AXPROP_DEVICE_3:

End Function


' Invoke configuration functions for device 4
Function INSTALL_DEVICE_4 as long

	dim PNUM as long

	INSTALL_DEVICE_4 = 0

	if Set_PDOe_To_AxProps_Device_4 = -1 then
		INSTALL_DEVICE_4 = -1
		goto INSTALL_DEVICE_4_DONE
	end if

	if SDO_Config_Device_4 = -1 then
		INSTALL_DEVICE_4 = -1
		goto INSTALL_DEVICE_4_DONE
	end if

	' Calculate Feedback Resolution
	PNUM = EC_SDO_READ(4, 0x6092, 1)
	if PNUM < 10000 then
		print "WARNING: PNUM is smaller than 10000. Low resolution might cause noises"
	end if
	call EC_USE_SLAVE(4, 0x2E1, 0x0, "ECAT_AXIS_SLAVE", TRUE)

	call EC_ASSOCIATE_PDO_TO_SYS_DIO(4, 0x60FD, 0, 4000, 11, 16)
	call EC_ASSOCIATE_PDO_TO_SYS_DIO(4, 0x60FE, 1, 4000, 6, 16)

INSTALL_DEVICE_4_DONE:

End Function


' Create PDO entries mapping of device 4
Function PDO_Mapping_Device_4 as long

	PDO_Mapping_Device_4 = 0

	' Zero number of entries in sync manager 2
	if Execute_VES("call EC_SDO_WRITE(4, EC_SYNC_MANAGER_1C12, 0, 8, 0)", 1107) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set PDOs in sync manager 2
	if Execute_VES("call EC_SDO_WRITE(4, EC_SYNC_MANAGER_1C12, 1, 16, 0x1600)", 1107) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	if Execute_VES("call EC_SDO_WRITE(4, EC_SYNC_MANAGER_1C12, 2, 16, 0x1601)", 1107) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	if Execute_VES("call EC_SDO_WRITE(4, EC_SYNC_MANAGER_1C12, 3, 16, 0x1603)", 1107) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set number of PDOs in the sync manager
	if Execute_VES("call EC_SDO_WRITE(4, EC_SYNC_MANAGER_1C12, 0, 8, 3)", 1107) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Zero number of PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(4, 0x1600, 0, 8, 0)", 1108) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(4, 0x1600, 1, 32, 0x60400010)", 1109) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	if Execute_VES("call EC_SDO_WRITE(4, 0x1600, 2, 32, 0x60600008)", 1110) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set number of PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(4, 0x1600, 0, 8, 2)", 1108) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Zero number of PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(4, 0x1601, 0, 8, 0)", 1111) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(4, 0x1601, 1, 32, 0x607A0020)", 1112) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	if Execute_VES("call EC_SDO_WRITE(4, 0x1601, 2, 32, 0x60810020)", 1113) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set number of PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(4, 0x1601, 0, 8, 2)", 1111) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Zero number of PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(4, 0x1603, 0, 8, 0)", 1114) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(4, 0x1603, 1, 32, 0x60FE0120)", 1115) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	if Execute_VES("call EC_SDO_WRITE(4, 0x1603, 2, 32, 0x60710010)", 1116) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	if Execute_VES("call EC_SDO_WRITE(4, 0x1603, 3, 32, 0x60B20010)", 1117) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set number of PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(4, 0x1603, 0, 8, 3)", 1114) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Zero number of entries in sync manager 3
	if Execute_VES("call EC_SDO_WRITE(4, EC_SYNC_MANAGER_1C13, 0, 8, 0)", 1118) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set PDOs in sync manager 3
	if Execute_VES("call EC_SDO_WRITE(4, EC_SYNC_MANAGER_1C13, 1, 16, 0x1A00)", 1118) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	if Execute_VES("call EC_SDO_WRITE(4, EC_SYNC_MANAGER_1C13, 2, 16, 0x1A01)", 1118) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	if Execute_VES("call EC_SDO_WRITE(4, EC_SYNC_MANAGER_1C13, 3, 16, 0x1A02)", 1118) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	if Execute_VES("call EC_SDO_WRITE(4, EC_SYNC_MANAGER_1C13, 4, 16, 0x1A03)", 1118) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set number of PDOs in the sync manager
	if Execute_VES("call EC_SDO_WRITE(4, EC_SYNC_MANAGER_1C13, 0, 8, 4)", 1118) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Zero number of PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(4, 0x1A00, 0, 8, 0)", 1119) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(4, 0x1A00, 1, 32, 0x60410010)", 1120) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	if Execute_VES("call EC_SDO_WRITE(4, 0x1A00, 2, 32, 0x60610008)", 1121) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	if Execute_VES("call EC_SDO_WRITE(4, 0x1A00, 3, 32, 0x60640020)", 1122) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set number of PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(4, 0x1A00, 0, 8, 3)", 1119) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Zero number of PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(4, 0x1A01, 0, 8, 0)", 1123) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(4, 0x1A01, 1, 32, 0x60640020)", 1124) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set number of PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(4, 0x1A01, 0, 8, 1)", 1123) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Zero number of PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(4, 0x1A02, 0, 8, 0)", 1125) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(4, 0x1A02, 1, 32, 0x60740010)", 1126) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	if Execute_VES("call EC_SDO_WRITE(4, 0x1A02, 2, 32, 0x20F20010)", 1127) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	if Execute_VES("call EC_SDO_WRITE(4, 0x1A02, 3, 32, 0x20F90010)", 1128) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	if Execute_VES("call EC_SDO_WRITE(4, 0x1A02, 4, 32, 0x60780010)", 1129) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set number of PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(4, 0x1A02, 0, 8, 4)", 1125) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Zero number of PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(4, 0x1A03, 0, 8, 0)", 1130) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(4, 0x1A03, 1, 32, 0x60FD0020)", 1131) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	if Execute_VES("call EC_SDO_WRITE(4, 0x1A03, 2, 32, 0x20B60020)", 1132) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	if Execute_VES("call EC_SDO_WRITE(4, 0x1A03, 3, 32, 0x60F40020)", 1133) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	' Set number of PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(4, 0x1A03, 0, 8, 3)", 1130) <> 0 then
		PDO_Mapping_Device_4 = -1
		goto ERR_PDO_DEVICE_4
	end if

	ETSLAVES[4]->et_opmode = 8

ERR_PDO_DEVICE_4:

End Function


' Configure device 4 via SDO writes
Function SDO_Config_Device_4 as long

	SDO_Config_Device_4 = 0

	print "PREOP configuration via SDO of device 4"
	print "Set MICRO_INTERPOLATION in device 4"
	if Execute_VES("call EC_SDO_WRITE(4, 0x60C0, 0, 16, 2)", 1134) <> 0 then
		SDO_Config_Device_4 = -1
		goto ERR_SDO_DEVICE_4
	end if

	print "Set PNUM in device 4"
	if Execute_VES("call EC_SDO_WRITE(4, 0x6092, 1, 32, 1000000)", 1135) <> 0 then
		SDO_Config_Device_4 = -1
		goto ERR_SDO_DEVICE_4
	end if

	print "Set PDEN in device 4"
	if Execute_VES("call EC_SDO_WRITE(4, 0x6092, 2, 32, 1)", 1136) <> 0 then
		SDO_Config_Device_4 = -1
		goto ERR_SDO_DEVICE_4
	end if

	print "Set GearRatio_MotorRevs in device 4"
	if Execute_VES("call EC_SDO_WRITE(4, 0x6091, 1, 32, 1)", 1137) <> 0 then
		SDO_Config_Device_4 = -1
		goto ERR_SDO_DEVICE_4
	end if

	print "Set GearRatio_ShaftRevs in device 4"
	if Execute_VES("call EC_SDO_WRITE(4, 0x6091, 2, 32, 1)", 1138) <> 0 then
		SDO_Config_Device_4 = -1
		goto ERR_SDO_DEVICE_4
	end if

	print "Set Smoothing in device 4"
	if Execute_VES("call EC_SDO_WRITE(4, 0x2109, 0, 16, 0)", 1139) <> 0 then
		SDO_Config_Device_4 = -1
		goto ERR_SDO_DEVICE_4
	end if

ERR_SDO_DEVICE_4:

End Function


' Configure device 4 via SDO writes
Function OpSDO_Config_Device_4 as long

	OpSDO_Config_Device_4 = 0

	print "OP configuration via SDO of device 4"
	print "Set DIGITAL_OUTPUTS_MASK in device 4"
	if Execute_VES("call EC_SDO_WRITE(4, 0x60FE, 2, 32, 0x3F0000)", 1140) <> 0 then
		OpSDO_Config_Device_4 = -1
		goto OpERR_SDO_DEVICE_4
	end if

OpERR_SDO_DEVICE_4:

End Function


' Associate PDO entries of device 4 with axis properties
Function Set_PDOe_To_AxProps_Device_4 as long

	Set_PDOe_To_AxProps_Device_4 = 0

	print "Associating between PDO entries of device 4 and axis properties"
	call EC_SET_CTRL_OBJECT(EC_BUSID, 4, EC_CTRL_INDEX_6040h, EC_CTRL_SUBINDEX_00h)

	call EC_SET_STATUS_OBJECT(EC_BUSID, 4, EC_STATUS_INDEX_6041h, EC_STATUS_SUBINDEX_00h)

	call EC_SET_PCMD_OBJECT(EC_BUSID, 4, EC_PCMD_INDEX_607Ah, EC_PCMD_SUBINDEX_00h)

	call EC_SET_PFB_OBJECT(EC_BUSID, 4, EC_PFB_INDEX_6064h, EC_PFB_SUBINDEX_00h)

	call EC_SET_VCMD_OBJECT(EC_BUSID, 4, EC_VCMD_INDEX_6081h, EC_VCMD_SUBINDEX_00h)

	call EC_SET_VADDCMD_OBJECT(EC_BUSID, 4, EC_VADDCMD_INDEX_60B1h, EC_VADDCMD_SUBINDEX_00h)

	call EC_SET_TRQCMD_OBJECT(EC_BUSID, 4, EC_TRQCMD_INDEX_6071h, EC_TRQCMD_SUBINDEX_00h)

	call EC_SET_TRQFBK_OBJECT(EC_BUSID, 4, EC_TRQFBK_INDEX_6078h, EC_TRQFBK_SUBINDEX_00h)

	call EC_SET_TRQDMND_OBJECT(EC_BUSID, 4, EC_TRQDMND_INDEX_6074h, EC_TRQDMND_SUBINDEX_00h)

	call EC_SET_TRQADDCMD_OBJECT(EC_BUSID, 4, EC_TRQADDCMD_INDEX_60B2h, EC_TRQADDCMD_SUBINDEX_00h)

	call EC_SET_PEXT_OBJECT(EC_BUSID, 4, EC_PEXT_INDEX_20B6h, EC_PEXT_SUBINDEX_00h)

ERR_PDOE_AXPROP_DEVICE_4:

End Function


' Invoke configuration functions for device 5
Function INSTALL_DEVICE_5 as long

	dim PNUM as long

	INSTALL_DEVICE_5 = 0

	if Set_PDOe_To_AxProps_Device_5 = -1 then
		INSTALL_DEVICE_5 = -1
		goto INSTALL_DEVICE_5_DONE
	end if

	if SDO_Config_Device_5 = -1 then
		INSTALL_DEVICE_5 = -1
		goto INSTALL_DEVICE_5_DONE
	end if

	' Calculate Feedback Resolution
	PNUM = EC_SDO_READ(5, 0x6092, 1)
	if PNUM < 10000 then
		print "WARNING: PNUM is smaller than 10000. Low resolution might cause noises"
	end if
	call EC_USE_SLAVE(5, 0x2E1, 0x0, "ECAT_AXIS_SLAVE", TRUE)

	call EC_ASSOCIATE_PDO_TO_SYS_DIO(5, 0x60FD, 0, 5000, 11, 16)
	call EC_ASSOCIATE_PDO_TO_SYS_DIO(5, 0x60FE, 1, 5000, 6, 16)

INSTALL_DEVICE_5_DONE:

End Function


' Create PDO entries mapping of device 5
Function PDO_Mapping_Device_5 as long

	PDO_Mapping_Device_5 = 0

	' Zero number of entries in sync manager 2
	if Execute_VES("call EC_SDO_WRITE(5, EC_SYNC_MANAGER_1C12, 0, 8, 0)", 1143) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set PDOs in sync manager 2
	if Execute_VES("call EC_SDO_WRITE(5, EC_SYNC_MANAGER_1C12, 1, 16, 0x1600)", 1143) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	if Execute_VES("call EC_SDO_WRITE(5, EC_SYNC_MANAGER_1C12, 2, 16, 0x1601)", 1143) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	if Execute_VES("call EC_SDO_WRITE(5, EC_SYNC_MANAGER_1C12, 3, 16, 0x1603)", 1143) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set number of PDOs in the sync manager
	if Execute_VES("call EC_SDO_WRITE(5, EC_SYNC_MANAGER_1C12, 0, 8, 3)", 1143) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Zero number of PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(5, 0x1600, 0, 8, 0)", 1144) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(5, 0x1600, 1, 32, 0x60400010)", 1145) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	if Execute_VES("call EC_SDO_WRITE(5, 0x1600, 2, 32, 0x60600008)", 1146) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set number of PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(5, 0x1600, 0, 8, 2)", 1144) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Zero number of PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(5, 0x1601, 0, 8, 0)", 1147) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(5, 0x1601, 1, 32, 0x607A0020)", 1148) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	if Execute_VES("call EC_SDO_WRITE(5, 0x1601, 2, 32, 0x60810020)", 1149) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set number of PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(5, 0x1601, 0, 8, 2)", 1147) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Zero number of PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(5, 0x1603, 0, 8, 0)", 1150) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(5, 0x1603, 1, 32, 0x60FE0120)", 1151) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set number of PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(5, 0x1603, 0, 8, 1)", 1150) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Zero number of entries in sync manager 3
	if Execute_VES("call EC_SDO_WRITE(5, EC_SYNC_MANAGER_1C13, 0, 8, 0)", 1152) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set PDOs in sync manager 3
	if Execute_VES("call EC_SDO_WRITE(5, EC_SYNC_MANAGER_1C13, 1, 16, 0x1A00)", 1152) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	if Execute_VES("call EC_SDO_WRITE(5, EC_SYNC_MANAGER_1C13, 2, 16, 0x1A01)", 1152) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	if Execute_VES("call EC_SDO_WRITE(5, EC_SYNC_MANAGER_1C13, 3, 16, 0x1A02)", 1152) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	if Execute_VES("call EC_SDO_WRITE(5, EC_SYNC_MANAGER_1C13, 4, 16, 0x1A03)", 1152) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set number of PDOs in the sync manager
	if Execute_VES("call EC_SDO_WRITE(5, EC_SYNC_MANAGER_1C13, 0, 8, 4)", 1152) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Zero number of PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(5, 0x1A00, 0, 8, 0)", 1153) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(5, 0x1A00, 1, 32, 0x60410010)", 1154) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	if Execute_VES("call EC_SDO_WRITE(5, 0x1A00, 2, 32, 0x60610008)", 1155) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set number of PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(5, 0x1A00, 0, 8, 2)", 1153) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Zero number of PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(5, 0x1A01, 0, 8, 0)", 1156) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(5, 0x1A01, 1, 32, 0x60640020)", 1157) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set number of PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(5, 0x1A01, 0, 8, 1)", 1156) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Zero number of PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(5, 0x1A02, 0, 8, 0)", 1158) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(5, 0x1A02, 1, 32, 0x60740010)", 1159) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	if Execute_VES("call EC_SDO_WRITE(5, 0x1A02, 2, 32, 0x20F20010)", 1160) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	if Execute_VES("call EC_SDO_WRITE(5, 0x1A02, 3, 32, 0x20F90010)", 1161) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set number of PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(5, 0x1A02, 0, 8, 3)", 1158) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Zero number of PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(5, 0x1A03, 0, 8, 0)", 1162) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(5, 0x1A03, 1, 32, 0x60FD0020)", 1163) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	if Execute_VES("call EC_SDO_WRITE(5, 0x1A03, 2, 32, 0x20B60020)", 1164) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	if Execute_VES("call EC_SDO_WRITE(5, 0x1A03, 3, 32, 0x60F40020)", 1165) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	' Set number of PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(5, 0x1A03, 0, 8, 3)", 1162) <> 0 then
		PDO_Mapping_Device_5 = -1
		goto ERR_PDO_DEVICE_5
	end if

	ETSLAVES[5]->et_opmode = 8

ERR_PDO_DEVICE_5:

End Function


' Configure device 5 via SDO writes
Function SDO_Config_Device_5 as long

	SDO_Config_Device_5 = 0

	print "PREOP configuration via SDO of device 5"
	print "Set MICRO_INTERPOLATION in device 5"
	if Execute_VES("call EC_SDO_WRITE(5, 0x60C0, 0, 16, 2)", 1166) <> 0 then
		SDO_Config_Device_5 = -1
		goto ERR_SDO_DEVICE_5
	end if

	print "Set PNUM in device 5"
	if Execute_VES("call EC_SDO_WRITE(5, 0x6092, 1, 32, 1000000)", 1167) <> 0 then
		SDO_Config_Device_5 = -1
		goto ERR_SDO_DEVICE_5
	end if

	print "Set PDEN in device 5"
	if Execute_VES("call EC_SDO_WRITE(5, 0x6092, 2, 32, 1)", 1168) <> 0 then
		SDO_Config_Device_5 = -1
		goto ERR_SDO_DEVICE_5
	end if

	print "Set GearRatio_MotorRevs in device 5"
	if Execute_VES("call EC_SDO_WRITE(5, 0x6091, 1, 32, 1)", 1169) <> 0 then
		SDO_Config_Device_5 = -1
		goto ERR_SDO_DEVICE_5
	end if

	print "Set GearRatio_ShaftRevs in device 5"
	if Execute_VES("call EC_SDO_WRITE(5, 0x6091, 2, 32, 1)", 1170) <> 0 then
		SDO_Config_Device_5 = -1
		goto ERR_SDO_DEVICE_5
	end if

	print "Set Smoothing in device 5"
	if Execute_VES("call EC_SDO_WRITE(5, 0x2109, 0, 16, 0)", 1171) <> 0 then
		SDO_Config_Device_5 = -1
		goto ERR_SDO_DEVICE_5
	end if

ERR_SDO_DEVICE_5:

End Function


' Configure device 5 via SDO writes
Function OpSDO_Config_Device_5 as long

	OpSDO_Config_Device_5 = 0

	print "OP configuration via SDO of device 5"
	print "Set DIGITAL_OUTPUTS_MASK in device 5"
	if Execute_VES("call EC_SDO_WRITE(5, 0x60FE, 2, 32, 0x3F0000)", 1172) <> 0 then
		OpSDO_Config_Device_5 = -1
		goto OpERR_SDO_DEVICE_5
	end if

OpERR_SDO_DEVICE_5:

End Function


' Associate PDO entries of device 5 with axis properties
Function Set_PDOe_To_AxProps_Device_5 as long

	Set_PDOe_To_AxProps_Device_5 = 0

	print "Associating between PDO entries of device 5 and axis properties"
	call EC_SET_CTRL_OBJECT(EC_BUSID, 5, EC_CTRL_INDEX_6040h, EC_CTRL_SUBINDEX_00h)

	call EC_SET_STATUS_OBJECT(EC_BUSID, 5, EC_STATUS_INDEX_6041h, EC_STATUS_SUBINDEX_00h)

	call EC_SET_PCMD_OBJECT(EC_BUSID, 5, EC_PCMD_INDEX_607Ah, EC_PCMD_SUBINDEX_00h)

	call EC_SET_PFB_OBJECT(EC_BUSID, 5, EC_PFB_INDEX_6064h, EC_PFB_SUBINDEX_00h)

	call EC_SET_VCMD_OBJECT(EC_BUSID, 5, EC_VCMD_INDEX_6081h, EC_VCMD_SUBINDEX_00h)

	call EC_SET_VADDCMD_OBJECT(EC_BUSID, 5, EC_VADDCMD_INDEX_60B1h, EC_VADDCMD_SUBINDEX_00h)

	call EC_SET_TRQCMD_OBJECT(EC_BUSID, 5, EC_TRQCMD_INDEX_6071h, EC_TRQCMD_SUBINDEX_00h)

	call EC_SET_TRQFBK_OBJECT(EC_BUSID, 5, EC_TRQFBK_INDEX_6078h, EC_TRQFBK_SUBINDEX_00h)

	call EC_SET_TRQDMND_OBJECT(EC_BUSID, 5, EC_TRQDMND_INDEX_6074h, EC_TRQDMND_SUBINDEX_00h)

	call EC_SET_TRQADDCMD_OBJECT(EC_BUSID, 5, EC_TRQADDCMD_INDEX_60B2h, EC_TRQADDCMD_SUBINDEX_00h)

	call EC_SET_PEXT_OBJECT(EC_BUSID, 5, EC_PEXT_INDEX_20B6h, EC_PEXT_SUBINDEX_00h)

ERR_PDOE_AXPROP_DEVICE_5:

End Function


' Invoke configuration functions for device 6
Function INSTALL_DEVICE_6 as long

	dim PNUM as long

	INSTALL_DEVICE_6 = 0

	if Set_PDOe_To_AxProps_Device_6 = -1 then
		INSTALL_DEVICE_6 = -1
		goto INSTALL_DEVICE_6_DONE
	end if

	if SDO_Config_Device_6 = -1 then
		INSTALL_DEVICE_6 = -1
		goto INSTALL_DEVICE_6_DONE
	end if

	' Calculate Feedback Resolution
	PNUM = EC_SDO_READ(6, 0x6092, 1)
	if PNUM < 10000 then
		print "WARNING: PNUM is smaller than 10000. Low resolution might cause noises"
	end if
	call EC_USE_SLAVE(6, 0x2E1, 0x0, "ECAT_AXIS_SLAVE", TRUE)

	call EC_ASSOCIATE_PDO_TO_SYS_DIO(6, 0x60FD, 0, 6000, 11, 16)
	call EC_ASSOCIATE_PDO_TO_SYS_DIO(6, 0x60FE, 1, 6000, 6, 16)

INSTALL_DEVICE_6_DONE:

End Function


' Create PDO entries mapping of device 6
Function PDO_Mapping_Device_6 as long

	PDO_Mapping_Device_6 = 0

	' Zero number of entries in sync manager 2
	if Execute_VES("call EC_SDO_WRITE(6, EC_SYNC_MANAGER_1C12, 0, 8, 0)", 1175) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set PDOs in sync manager 2
	if Execute_VES("call EC_SDO_WRITE(6, EC_SYNC_MANAGER_1C12, 1, 16, 0x1600)", 1175) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	if Execute_VES("call EC_SDO_WRITE(6, EC_SYNC_MANAGER_1C12, 2, 16, 0x1601)", 1175) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	if Execute_VES("call EC_SDO_WRITE(6, EC_SYNC_MANAGER_1C12, 3, 16, 0x1603)", 1175) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set number of PDOs in the sync manager
	if Execute_VES("call EC_SDO_WRITE(6, EC_SYNC_MANAGER_1C12, 0, 8, 3)", 1175) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Zero number of PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(6, 0x1600, 0, 8, 0)", 1176) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(6, 0x1600, 1, 32, 0x60400010)", 1177) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	if Execute_VES("call EC_SDO_WRITE(6, 0x1600, 2, 32, 0x60600008)", 1178) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set number of PDO entries in PDO 0x1600
	if Execute_VES("call EC_SDO_WRITE(6, 0x1600, 0, 8, 2)", 1176) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Zero number of PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(6, 0x1601, 0, 8, 0)", 1179) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(6, 0x1601, 1, 32, 0x607A0020)", 1180) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	if Execute_VES("call EC_SDO_WRITE(6, 0x1601, 2, 32, 0x60810020)", 1181) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set number of PDO entries in PDO 0x1601
	if Execute_VES("call EC_SDO_WRITE(6, 0x1601, 0, 8, 2)", 1179) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Zero number of PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(6, 0x1603, 0, 8, 0)", 1182) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(6, 0x1603, 1, 32, 0x60FE0120)", 1183) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	if Execute_VES("call EC_SDO_WRITE(6, 0x1603, 2, 32, 0x60710010)", 1184) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	if Execute_VES("call EC_SDO_WRITE(6, 0x1603, 3, 32, 0x60B20010)", 1185) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set number of PDO entries in PDO 0x1603
	if Execute_VES("call EC_SDO_WRITE(6, 0x1603, 0, 8, 3)", 1182) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Zero number of entries in sync manager 3
	if Execute_VES("call EC_SDO_WRITE(6, EC_SYNC_MANAGER_1C13, 0, 8, 0)", 1186) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set PDOs in sync manager 3
	if Execute_VES("call EC_SDO_WRITE(6, EC_SYNC_MANAGER_1C13, 1, 16, 0x1A00)", 1186) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	if Execute_VES("call EC_SDO_WRITE(6, EC_SYNC_MANAGER_1C13, 2, 16, 0x1A01)", 1186) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	if Execute_VES("call EC_SDO_WRITE(6, EC_SYNC_MANAGER_1C13, 3, 16, 0x1A02)", 1186) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	if Execute_VES("call EC_SDO_WRITE(6, EC_SYNC_MANAGER_1C13, 4, 16, 0x1A03)", 1186) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set number of PDOs in the sync manager
	if Execute_VES("call EC_SDO_WRITE(6, EC_SYNC_MANAGER_1C13, 0, 8, 4)", 1186) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Zero number of PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(6, 0x1A00, 0, 8, 0)", 1187) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(6, 0x1A00, 1, 32, 0x60410010)", 1188) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	if Execute_VES("call EC_SDO_WRITE(6, 0x1A00, 2, 32, 0x60610008)", 1189) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	if Execute_VES("call EC_SDO_WRITE(6, 0x1A00, 3, 32, 0x60640020)", 1190) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set number of PDO entries in PDO 0x1A00
	if Execute_VES("call EC_SDO_WRITE(6, 0x1A00, 0, 8, 3)", 1187) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Zero number of PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(6, 0x1A01, 0, 8, 0)", 1191) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(6, 0x1A01, 1, 32, 0x60640020)", 1192) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set number of PDO entries in PDO 0x1A01
	if Execute_VES("call EC_SDO_WRITE(6, 0x1A01, 0, 8, 1)", 1191) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Zero number of PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(6, 0x1A02, 0, 8, 0)", 1193) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(6, 0x1A02, 1, 32, 0x60740010)", 1194) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	if Execute_VES("call EC_SDO_WRITE(6, 0x1A02, 2, 32, 0x20F20010)", 1195) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	if Execute_VES("call EC_SDO_WRITE(6, 0x1A02, 3, 32, 0x20F90010)", 1196) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	if Execute_VES("call EC_SDO_WRITE(6, 0x1A02, 4, 32, 0x60780010)", 1197) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set number of PDO entries in PDO 0x1A02
	if Execute_VES("call EC_SDO_WRITE(6, 0x1A02, 0, 8, 4)", 1193) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Zero number of PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(6, 0x1A03, 0, 8, 0)", 1198) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(6, 0x1A03, 1, 32, 0x60FD0020)", 1199) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	if Execute_VES("call EC_SDO_WRITE(6, 0x1A03, 2, 32, 0x20B60020)", 1200) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	if Execute_VES("call EC_SDO_WRITE(6, 0x1A03, 3, 32, 0x60F40020)", 1201) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	' Set number of PDO entries in PDO 0x1A03
	if Execute_VES("call EC_SDO_WRITE(6, 0x1A03, 0, 8, 3)", 1198) <> 0 then
		PDO_Mapping_Device_6 = -1
		goto ERR_PDO_DEVICE_6
	end if

	ETSLAVES[6]->et_opmode = 8

ERR_PDO_DEVICE_6:

End Function


' Configure device 6 via SDO writes
Function SDO_Config_Device_6 as long

	SDO_Config_Device_6 = 0

	print "PREOP configuration via SDO of device 6"
	print "Set MICRO_INTERPOLATION in device 6"
	if Execute_VES("call EC_SDO_WRITE(6, 0x60C0, 0, 16, 2)", 1202) <> 0 then
		SDO_Config_Device_6 = -1
		goto ERR_SDO_DEVICE_6
	end if

	print "Set PNUM in device 6"
	if Execute_VES("call EC_SDO_WRITE(6, 0x6092, 1, 32, 1000000)", 1203) <> 0 then
		SDO_Config_Device_6 = -1
		goto ERR_SDO_DEVICE_6
	end if

	print "Set PDEN in device 6"
	if Execute_VES("call EC_SDO_WRITE(6, 0x6092, 2, 32, 1)", 1204) <> 0 then
		SDO_Config_Device_6 = -1
		goto ERR_SDO_DEVICE_6
	end if

	print "Set GearRatio_MotorRevs in device 6"
	if Execute_VES("call EC_SDO_WRITE(6, 0x6091, 1, 32, 1)", 1205) <> 0 then
		SDO_Config_Device_6 = -1
		goto ERR_SDO_DEVICE_6
	end if

	print "Set GearRatio_ShaftRevs in device 6"
	if Execute_VES("call EC_SDO_WRITE(6, 0x6091, 2, 32, 1)", 1206) <> 0 then
		SDO_Config_Device_6 = -1
		goto ERR_SDO_DEVICE_6
	end if

	print "Set Smoothing in device 6"
	if Execute_VES("call EC_SDO_WRITE(6, 0x2109, 0, 16, 0)", 1207) <> 0 then
		SDO_Config_Device_6 = -1
		goto ERR_SDO_DEVICE_6
	end if

ERR_SDO_DEVICE_6:

End Function


' Configure device 6 via SDO writes
Function OpSDO_Config_Device_6 as long

	OpSDO_Config_Device_6 = 0

	print "OP configuration via SDO of device 6"
	print "Set DIGITAL_OUTPUTS_MASK in device 6"
	if Execute_VES("call EC_SDO_WRITE(6, 0x60FE, 2, 32, 0x3F0000)", 1208) <> 0 then
		OpSDO_Config_Device_6 = -1
		goto OpERR_SDO_DEVICE_6
	end if

OpERR_SDO_DEVICE_6:

End Function


' Associate PDO entries of device 6 with axis properties
Function Set_PDOe_To_AxProps_Device_6 as long

	Set_PDOe_To_AxProps_Device_6 = 0

	print "Associating between PDO entries of device 6 and axis properties"
	call EC_SET_CTRL_OBJECT(EC_BUSID, 6, EC_CTRL_INDEX_6040h, EC_CTRL_SUBINDEX_00h)

	call EC_SET_STATUS_OBJECT(EC_BUSID, 6, EC_STATUS_INDEX_6041h, EC_STATUS_SUBINDEX_00h)

	call EC_SET_PCMD_OBJECT(EC_BUSID, 6, EC_PCMD_INDEX_607Ah, EC_PCMD_SUBINDEX_00h)

	call EC_SET_PFB_OBJECT(EC_BUSID, 6, EC_PFB_INDEX_6064h, EC_PFB_SUBINDEX_00h)

	call EC_SET_VCMD_OBJECT(EC_BUSID, 6, EC_VCMD_INDEX_6081h, EC_VCMD_SUBINDEX_00h)

	call EC_SET_VADDCMD_OBJECT(EC_BUSID, 6, EC_VADDCMD_INDEX_60B1h, EC_VADDCMD_SUBINDEX_00h)

	call EC_SET_TRQCMD_OBJECT(EC_BUSID, 6, EC_TRQCMD_INDEX_6071h, EC_TRQCMD_SUBINDEX_00h)

	call EC_SET_TRQFBK_OBJECT(EC_BUSID, 6, EC_TRQFBK_INDEX_6078h, EC_TRQFBK_SUBINDEX_00h)

	call EC_SET_TRQDMND_OBJECT(EC_BUSID, 6, EC_TRQDMND_INDEX_6074h, EC_TRQDMND_SUBINDEX_00h)

	call EC_SET_TRQADDCMD_OBJECT(EC_BUSID, 6, EC_TRQADDCMD_INDEX_60B2h, EC_TRQADDCMD_SUBINDEX_00h)

	call EC_SET_PEXT_OBJECT(EC_BUSID, 6, EC_PEXT_INDEX_20B6h, EC_PEXT_SUBINDEX_00h)

ERR_PDOE_AXPROP_DEVICE_6:

End Function


function Execute_VES(byval command as string, byval err_code as long) as long

	dim retStr as string

	Execute_VES = 0
	EC_SETUP_PRG_error_code = 0
	retStr = VesExecute(command)

	if INSTR(retStr, "E:Error") <> 0  then

		EC_SETUP_PRG_error_code = err_code
		EC_SETUP_PRG_error_str = retStr
		Execute_VES = -1

	end if

end function

