'------------------------------------------------------------------------------
' File:        Helix.prg
' Purpose:     generating puma helix motion
' Version:     1.00
' Author:      Eran Korkidi
' History:     10.DEC.2015    -    created
'------------------------------------------------------------------------------

' module global "constants"

 
' module global variables

dim shared ZeroPosition as joint of xyzypr = {0,-90,180,0,90,0}                      '{J1,J2,J3,J4,J5,J6}
 
dim shared HelixCenter as location of xyzypr = #{500 , 0 , 250 ,0 , -180 , 0}        ' {X,Y,Z,Y,P,R} - Center end point of Helix 
dim shared HelixStart as location of xyzypr =  #{500 , 100 , 50 , 0 , -180 , 0}      ' starting point of Helix - on the circle - the Y coordinate is the radius of the circle
 
dim shared RobotVelocity as double = 40.0
 
program
 
    Sys.Vrate = 100.0                        'System's velocity factor (in percents). 100 means no factor.
    with Puma01
        Attach
            En = TRUE
            Sleep 100
            while NOT En
                Sleep 100
            Vcruise = RobotVelocity          'Set robot's cruise velocity 
            end while
            Move ZeroPosition
            WaitForMotion Puma01
                call helixMotion
            Move ZeroPosition
            WaitForMotion Puma01
        Detach
    end with    
 
end program
 
 
sub helixMotion
 
    dim i as long
 
    PUMA01.BlendingMethod = 1               'Type of motion blending - 1 is Continous Path blending.
    PUMA01.Cp = 2                           'At continous path blending, sets the blend radius value - bigger equal smoother
    Move PUMA01 HelixStart
    WaitForMotion Puma01
    for i = 1 to 3
         Circle PUMA01 Angle=3*360 CircleCenter = HelixCenter Vtran = PUMA01.VmTran     'Circle center has different Z coordinate than circle starting point - causing Helix and not plane circle
         Moves PUMA01 HelixStart Vtran=PUMA01.VmTran                                    'Moves to start point in straight line (translation)
    next i
 
end sub